from thermostat.importers import from_csv
from thermostat.importers import get_single_thermostat
from thermostat.util.testing import get_data_path
from thermostat.core import Thermostat, CoreDaySet
from tempfile import TemporaryDirectory

import pandas as pd
import numpy as np
from numpy import nan

import pytest

''' Abbreviations used in this file:

    XX - Heating system type: fu (furnace), hp (heat pump),  er (electric resistance), ot (other)
    XX - Cooling system type: ce (central), hp (heat pump), 

    YY - backup: eb (electric backup), ne (non electric backup, df (dual fuel), na (N/A)

    Z - speed: 1 (single stage), 2 (two stage), v (variable)
'''

@pytest.fixture(scope="session", params=["../data/two_stage_ert/metadata_furnace_or_boiler_two_stage_central_two_stage.csv"])
def thermostat_ert_fu_2_ce_2(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/two_stage_ert/metadata_furnace_or_boiler_two_stage_none_single_stage.csv"])
def thermostat_ert_fu_2_na_1(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/two_stage_ert/metadata_heat_pump_electric_backup_two_stage_heat_pump_two_stage.csv"])
def thermostat_ert_hpeb_2_hp_2(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/two_stage_ert/metadata_none_two_stage_heat_pump_two_stage.csv"])
def thermostat_ert_na_2_hp_2(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session")
def core_heating_day_set_ert_hpeb_2_hp_2_entire(thermostat_ert_hpeb_2_hp_2):
    return thermostat_ert_hpeb_2_hp_2.get_core_heating_days(method="entire_dataset")[0]

@pytest.fixture(scope="session")
def core_cooling_day_set_ert_hpeb_2_hp_2_entire(thermostat_ert_hpeb_2_hp_2):
    return thermostat_ert_hpeb_2_hp_2.get_core_cooling_days(method="entire_dataset")[0]


@pytest.fixture(scope="session")
def metrics_ert_hpeb_2_hp_2_data():
    data = [{'sw_version': '2.0.0a1',
	'ct_identifier': 'c61badb0e0c0a7e06932de804af43111',
	'heat_type': 'heat_pump_electric_backup',
	'heat_stage': 'two_stage',
	'cool_type': 'heat_pump',
	'cool_stage': 'two_stage',
	'heating_or_cooling': 'cooling_ALL',
	'zipcode': '27313',
	'station': '723170',
	'climate_zone': 'Mixed-Humid',
	'start_date': '2018-01-01T00:00:00',
	'end_date': '2018-12-31T00:00:00',
	'n_days_in_inputfile_date_range': 364,
	'n_days_both_heating_and_cooling': 17,
	'n_days_insufficient_data': 51,
	'n_core_cooling_days': 159,
	'baseline_percentile_core_cooling_comfort_temperature': 70.9,
	'regional_average_baseline_cooling_comfort_temperature': 73.0,
	'percent_savings_baseline_percentile': 7.949942200976018,
	'avoided_daily_mean_core_day_runtime_baseline_percentile': 26.653401651166952,
	'avoided_total_core_day_runtime_baseline_percentile': 4237.890862535545,
	'baseline_daily_mean_core_day_runtime_baseline_percentile': 335.26535133670154,
	'baseline_total_core_day_runtime_baseline_percentile': 53307.19086253554,
	'_daily_mean_core_day_demand_baseline_baseline_percentile': 12.168302421738765,
	'percent_savings_baseline_regional': -9.462769099731771,
	'avoided_daily_mean_core_day_runtime_baseline_regional': -26.678693087249965,
	'avoided_total_core_day_runtime_baseline_regional': -4241.9122008727445,
	'baseline_daily_mean_core_day_runtime_baseline_regional': 281.93325659828463,
	'baseline_total_core_day_runtime_baseline_regional': 44827.38779912725,
	'_daily_mean_core_day_demand_baseline_baseline_regional': 10.232638461909708,
	'mean_demand': 11.200929412370568,
	'tau': 8.371282818608988,
	'alpha': 27.55235198113974,
	'mean_sq_err': 2714.7188218629863,
	'root_mean_sq_err': 52.10296365719503,
	'cv_root_mean_sq_err': 0.16883002654396964,
	'mean_abs_pct_err': 0.1270082906243978,
	'mean_abs_err': 39.19627619582243,
	'total_core_cooling_runtime': 49069.3,
	'daily_mean_core_cooling_runtime': 308.6119496855346,
	'core_cooling_days_mean_indoor_temperature': 71.87575995807128,
	'core_cooling_days_mean_outdoor_temperature': 74.44325500000001,
	'core_mean_indoor_temperature': 71.87575995807128,
	'core_mean_outdoor_temperature': 74.44325500000001},
	{'sw_version': '2.0.0a1',
	'ct_identifier': 'c61badb0e0c0a7e06932de804af43111',
	'heat_type': 'heat_pump_electric_backup',
	'heat_stage': 'two_stage',
	'cool_type': 'heat_pump',
	'cool_stage': 'two_stage',
	'heating_or_cooling': 'heating_ALL',
	'zipcode': '27313',
	'station': '723170',
	'climate_zone': 'Mixed-Humid',
	'start_date': '2018-01-01T00:00:00',
	'end_date': '2018-12-31T00:00:00',
	'n_days_in_inputfile_date_range': 364,
	'n_days_both_heating_and_cooling': 17,
	'n_days_insufficient_data': 51,
	'n_core_heating_days': 116,
	'baseline_percentile_core_heating_comfort_temperature': 70.0,
	'regional_average_baseline_heating_comfort_temperature': 69,
	'percent_savings_baseline_percentile': 9.864301763590625,
	'avoided_daily_mean_core_day_runtime_baseline_percentile': 32.91716771902953,
	'avoided_total_core_day_runtime_baseline_percentile': 3818.391455407425,
	'baseline_daily_mean_core_day_runtime_baseline_percentile': 333.6999263397191,
	'baseline_total_core_day_runtime_baseline_percentile': 38709.19145540742,
	'_daily_mean_core_day_demand_baseline_baseline_percentile': 13.175487715848755,
	'percent_savings_baseline_regional': 3.31499473314762,
	'avoided_daily_mean_core_day_runtime_baseline_regional': 10.312801430761704,
	'avoided_total_core_day_runtime_baseline_regional': 1196.2849659683577,
	'baseline_daily_mean_core_day_runtime_baseline_regional': 311.09556005145134,
	'baseline_total_core_day_runtime_baseline_regional': 36087.08496596836,
	'_daily_mean_core_day_demand_baseline_baseline_regional': 12.2829986055802,
	'mean_demand': 11.875817848732618,
	'tau': 13.624549353183188,
	'alpha': 25.327330079653336,
	'mean_sq_err': 5950.806958626368,
	'root_mean_sq_err': 77.14147366123082,
	'cv_root_mean_sq_err': 0.25646906762535615,
	'mean_abs_pct_err': 0.1972712328788015,
	'mean_abs_err': 59.33578562179039,
	'total_core_heating_runtime': 34890.799999999996,
	'daily_mean_core_heating_runtime': 300.7827586206896,
	'core_heating_days_mean_indoor_temperature': 68.65607040229885,
	'core_heating_days_mean_outdoor_temperature': 43.64093413793104,
	'core_mean_indoor_temperature': 68.65607040229885,
	'core_mean_outdoor_temperature': 43.64093413793104,
	'total_auxiliary_heating_core_day_runtime': 1512.6000000000001,
	'total_emergency_heating_core_day_runtime': 0.0,
	'rhu1_00F_to_05F': nan,
	'rhu1_05F_to_10F': nan,
	'rhu1_10F_to_15F': nan,
	'rhu1_15F_to_20F': nan,
	'rhu1_20F_to_25F': 0.6045587903407809,
	'rhu1_25F_to_30F': 0.12034338820193748,
	'rhu1_30F_to_35F': 0.0771375321534741,
	'rhu1_35F_to_40F': 0.04561882032049096,
	'rhu1_40F_to_45F': 0.03743520199479428,
	'rhu1_45F_to_50F': 0.014363535770523898,
	'rhu1_50F_to_55F': 0.0,
	'rhu1_55F_to_60F': 0.015546218487394958,
	'rhu1_30F_to_45F': 0.050930833104836905,
	'rhu2_00F_to_05F': nan,
	'rhu2_05F_to_10F': nan,
	'rhu2_10F_to_15F': nan,
	'rhu2_15F_to_20F': nan,
	'rhu2_20F_to_25F': 0.6045587903407809,
	'rhu2_25F_to_30F': nan,
	'rhu2_30F_to_35F': 0.0771375321534741,
	'rhu2_35F_to_40F': 0.04561882032049096,
	'rhu2_40F_to_45F': 0.03743520199479428,
	'rhu2_45F_to_50F': 0.014363535770523898,
	'rhu2_50F_to_55F': 0.0,
	'rhu2_55F_to_60F': nan,
	'rhu2_30F_to_45F': 0.050930833104836905}]


    return data



from thermostat.importers import from_csv
from thermostat.importers import get_single_thermostat
from thermostat.util.testing import get_data_path
from thermostat.core import Thermostat, CoreDaySet
from tempfile import TemporaryDirectory

import pandas as pd
import numpy as np
from numpy import nan

import pytest

# Note:
# The following fixtures can be quite slow without a prebuilt weather cache
# they the from_csv command fetches weather data. (This happens with builds on
# travis.)
# To speed this up, spoof the weather source.

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_1_single_utc_offset_0.csv"])
def thermostat_type_1_utc(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_1_single_utc_offset_bad.csv"])
def thermostat_type_1_utc_bad(request):
    thermostats = from_csv(get_data_path(request.param))

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_1_single_bad_zip.csv"])
def thermostat_type_1_zip_bad(request):
    thermostats = from_csv(get_data_path(request.param))
    return list(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_multiple_same_key.csv"])
def thermostats_multiple_same_key(request):
    thermostats = from_csv(get_data_path(request.param))
    return thermostats

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_1_single_too_many_minutes.csv"])
def thermostat_type_1_too_many_minutes(request):
    thermostats = from_csv(get_data_path(request.param))
    return list(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_1_single_data_out_of_order.csv"])
def thermostat_type_1_data_out_of_order(request):
    thermostats = from_csv(get_data_path(request.param))
    return list(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_1_single.csv"])
def thermostat_type_1(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_1_single.csv"])
def thermostat_type_1_cache(request):
    with TemporaryDirectory() as tempdir:
        thermostats = from_csv(get_data_path(request.param), save_cache=True, cache_path=tempdir)
        return next(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_2_single.csv"])
def thermostat_type_2(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_3_single.csv"])
def thermostat_type_3(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_4_single.csv"])
def thermostat_type_4(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_type_5_single.csv"])
def thermostat_type_5(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_single_zero_days.csv"])
def thermostat_zero_days(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/single_stage/metadata_single_emg_aux_constant_on_outlier.csv"])
def thermostat_emg_aux_constant_on_outlier(request):
    thermostats = from_csv(get_data_path(request.param))
    return thermostats

@pytest.fixture(scope="session")
def core_heating_day_set_type_1_mid_to_mid(thermostat_type_1):
    return thermostat_type_1.get_core_heating_days(method="year_mid_to_mid")[0]

@pytest.fixture(scope="session")
def core_heating_day_set_type_1_entire(thermostat_type_1):
    return thermostat_type_1.get_core_heating_days(method="entire_dataset")[0]

@pytest.fixture(scope="session")
def core_heating_day_set_type_2(thermostat_type_2):
    return thermostat_type_2.get_core_heating_days(method="year_mid_to_mid")[0]

@pytest.fixture(scope="session")
def core_heating_day_set_type_3(thermostat_type_3):
    return thermostat_type_3.get_core_heating_days(method="year_mid_to_mid")[0]

@pytest.fixture(scope="session")
def core_heating_day_set_type_4(thermostat_type_4):
    return thermostat_type_4.get_core_heating_days(method="year_mid_to_mid")[0]

@pytest.fixture(scope="session")
def core_cooling_day_set_type_1_end_to_end(thermostat_type_1):
    return thermostat_type_1.get_core_cooling_days(method="year_end_to_end")[0]

@pytest.fixture(scope="session")
def core_cooling_day_set_type_1_entire(thermostat_type_1):
    return thermostat_type_1.get_core_cooling_days(method="entire_dataset")[0]

@pytest.fixture(scope="session")
def core_cooling_day_set_type_1_empty(thermostat_type_1):
    core_cooling_day_set = thermostat_type_1.get_core_cooling_days(method="entire_dataset")[0]
    core_day_set = CoreDaySet(
        "empty",
        pd.Series(np.tile(False, core_cooling_day_set.daily.shape),
                  index=core_cooling_day_set.daily.index),
        pd.Series(np.tile(False, core_cooling_day_set.hourly.shape),
                  index=core_cooling_day_set.hourly.index),
        core_cooling_day_set.start_date,
        core_cooling_day_set.end_date
    )
    return core_day_set

@pytest.fixture(scope="session")
def core_heating_day_set_type_1_empty(thermostat_type_1):
    core_heating_day_set = thermostat_type_1.get_core_heating_days(method="entire_dataset")[0]
    core_day_set = CoreDaySet(
        "empty",
        pd.Series(np.tile(False, core_heating_day_set.daily.shape),
                  index=core_heating_day_set.daily.index),
        pd.Series(np.tile(False, core_heating_day_set.hourly.shape),
                  index=core_heating_day_set.hourly.index),
        core_heating_day_set.start_date,
        core_heating_day_set.end_date
    )
    return core_day_set

@pytest.fixture(scope="session")
def core_cooling_day_set_type_2(thermostat_type_2):
    return thermostat_type_2.get_core_cooling_days(method="year_end_to_end")[0]

@pytest.fixture(scope="session")
def core_cooling_day_set_type_3(thermostat_type_3):
    return thermostat_type_3.get_core_cooling_days(method="year_end_to_end")[0]

@pytest.fixture(scope="session")
def core_cooling_day_set_type_5(thermostat_type_5):
    return thermostat_type_5.get_core_cooling_days(method="year_end_to_end")[0]

@pytest.fixture(scope="session")
def metrics_type_1_data():

    # this data comes from a script in scripts/test_data_generation.ipynb
    data = [{'sw_version': '2.0.0a1',
        'ct_identifier': '8465829e-df0d-449e-97bf-96317c24dec3',
        'heat_type': 'heat_pump_electric_backup',
        'heat_stage': 'single_stage',
        'cool_type': 'heat_pump',
        'cool_stage': '',
        'heating_or_cooling': 'cooling_ALL',
        'zipcode': '62223',
        'station': '725314',
        'climate_zone': 'Mixed-Humid',
        'start_date': '2011-01-01T00:00:00',
        'end_date': '2014-12-31T00:00:00',
        'n_days_in_inputfile_date_range': 1460,
        'n_days_both_heating_and_cooling': 212,
        'n_days_insufficient_data': 13,
        'n_core_cooling_days': 296,
        'baseline_percentile_core_cooling_comfort_temperature': 69.5,
        'regional_average_baseline_cooling_comfort_temperature': 73.0,
        'percent_savings_baseline_percentile': 43.83576239614899,
        'avoided_daily_mean_core_day_runtime_baseline_percentile': 192.71571072318235,
        'avoided_total_core_day_runtime_baseline_percentile': 57043.85037406198,
        'baseline_daily_mean_core_day_runtime_baseline_percentile': 439.63125126372296,
        'baseline_total_core_day_runtime_baseline_percentile': 130130.85037406199,
        '_daily_mean_core_day_demand_baseline_baseline_percentile': 9.97608214479391,
        'percent_savings_baseline_regional': 21.40352148762093,
        'avoided_daily_mean_core_day_runtime_baseline_regional': 67.24044356204358,
        'avoided_total_core_day_runtime_baseline_regional': 19903.1712943649,
        'baseline_daily_mean_core_day_runtime_baseline_regional': 314.1559841025841,
        'baseline_total_core_day_runtime_baseline_regional': 92990.1712943649,
        '_daily_mean_core_day_demand_baseline_baseline_regional': 7.128806004298179,
        'mean_demand': 5.602990479357406,
        'tau': -0.8165326579183203,
        'alpha': 44.06852759258279,
        'mean_sq_err': 379.2822470522948,
        'root_mean_sq_err': 19.47517001343749,
        'cv_root_mean_sq_err': 0.07887381236030344,
        'mean_abs_pct_err': 0.05040045923258922,
        'mean_abs_err': 12.444656634906245,
        'total_core_cooling_runtime': 73087.0,
        'daily_mean_core_cooling_runtime': 246.91554054054055,
        'core_cooling_days_mean_indoor_temperature': 73.95971753003002,
        'core_cooling_days_mean_outdoor_temperature': 79.8426321875,
        'core_mean_indoor_temperature': 73.95971753003002,
        'core_mean_outdoor_temperature': 79.8426321875},
        {'sw_version': '2.0.0a1',
        'ct_identifier': '8465829e-df0d-449e-97bf-96317c24dec3',
        'heat_type': 'heat_pump_electric_backup',
        'heat_stage': 'single_stage',
        'cool_type': 'heat_pump',
        'cool_stage': '',
        'heating_or_cooling': 'heating_ALL',
        'zipcode': '62223',
        'station': '725314',
        'climate_zone': 'Mixed-Humid',
        'start_date': '2011-01-01T00:00:00',
        'end_date': '2014-12-31T00:00:00',
        'n_days_in_inputfile_date_range': 1460,
        'n_days_both_heating_and_cooling': 212,
        'n_days_insufficient_data': 13,
        'n_core_heating_days': 895,
        'baseline_percentile_core_heating_comfort_temperature': 69.5,
        'regional_average_baseline_heating_comfort_temperature': 69,
        'percent_savings_baseline_percentile': 10.646928880334725,
        'avoided_daily_mean_core_day_runtime_baseline_percentile': 92.11668543737152,
        'avoided_total_core_day_runtime_baseline_percentile': 82444.4334664475,
        'baseline_daily_mean_core_day_runtime_baseline_percentile': 865.1948977278741,
        'baseline_total_core_day_runtime_baseline_percentile': 774349.4334664474,
        '_daily_mean_core_day_demand_baseline_baseline_percentile': 27.29246198483337,
        'percent_savings_baseline_regional': 9.033255766077659,
        'avoided_daily_mean_core_day_runtime_baseline_regional': 76.7688596268131,
        'avoided_total_core_day_runtime_baseline_regional': 68708.12936599772,
        'baseline_daily_mean_core_day_runtime_baseline_regional': 849.8470719173159,
        'baseline_total_core_day_runtime_baseline_regional': 760613.1293659977,
        '_daily_mean_core_day_demand_baseline_baseline_regional': 26.80831678982061,
        'mean_demand': 24.38665296761577,
        'tau': -2.348660688368403,
        'alpha': 31.70087397057362,
        'mean_sq_err': 6897.070484921533,
        'root_mean_sq_err': 83.04860314852702,
        'cv_root_mean_sq_err': 0.10742587467633805,
        'mean_abs_pct_err': 0.0719217893742982,
        'mean_abs_err': 55.60116835421654,
        'total_core_heating_runtime': 691905.0,
        'daily_mean_core_heating_runtime': 773.0782122905028,
        'core_heating_days_mean_indoor_temperature': 66.6958723285375,
        'core_heating_days_mean_outdoor_temperature': 44.679677289106145,
        'core_mean_indoor_temperature': 66.6958723285375,
        'core_mean_outdoor_temperature': 44.679677289106145,
        'total_auxiliary_heating_core_day_runtime': 144794.0,
        'total_emergency_heating_core_day_runtime': 2104.0,
        'rhu1_00F_to_05F': nan,
        'rhu1_05F_to_10F': 0.35810185185185184,
        'rhu1_10F_to_15F': 0.36336805555555557,
        'rhu1_15F_to_20F': 0.37900691389063484,
        'rhu1_20F_to_25F': 0.375673443706005,
        'rhu1_25F_to_30F': 0.3318400322150354,
        'rhu1_30F_to_35F': 0.28432496802364043,
        'rhu1_35F_to_40F': 0.19741898266605878,
        'rhu1_40F_to_45F': 0.15271363589013506,
        'rhu1_45F_to_50F': 0.09249776186213071,
        'rhu1_50F_to_55F': 0.052322643343051506,
        'rhu1_55F_to_60F': 0.028319891645631964,
        'rhu1_30F_to_45F': 0.22154695797667256,
        'rhu2_00F_to_05F': nan,
        'rhu2_05F_to_10F': 0.35810185185185184,
        'rhu2_10F_to_15F': 0.36336805555555557,
        'rhu2_15F_to_20F': 0.37900691389063484,
        'rhu2_20F_to_25F': 0.375673443706005,
        'rhu2_25F_to_30F': 0.3318400322150354,
        'rhu2_30F_to_35F': 0.28432496802364043,
        'rhu2_35F_to_40F': 0.19741898266605878,
        'rhu2_40F_to_45F': 0.15271363589013506,
        'rhu2_45F_to_50F': 0.09249776186213071,
        'rhu2_50F_to_55F': 0.052322643343051506,
        'rhu2_55F_to_60F': 0.028319891645631964,
        'rhu2_30F_to_45F': 0.22154695797667256}]

    return data



from thermostat.importers import from_csv
from thermostat.importers import get_single_thermostat
from thermostat.util.testing import get_data_path
from thermostat.core import Thermostat, CoreDaySet
from tempfile import TemporaryDirectory

import pandas as pd
import numpy as np
from numpy import nan

import pytest

''' Abbreviations used in this file:

    XX - Heating system type: fu (furnace), hp (heat pump),  er (electric resistance), ot (other)
    XX - Cooling system type: ce (central), hp (heat pump), 

    YY - backup: eb (electric backup), ne (non electric backup, df (dual fuel), na (N/A)

    Z - speed: 1 (single stage), 2 (two stage), v (variable)
'''

@pytest.fixture(scope="session", params=["../data/two_stage/metadata_furnace_or_boiler_two_stage_central_two_stage.csv"])
def thermostat_fu_2_ce_2(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/two_stage/metadata_furnace_or_boiler_two_stage_none_single_stage.csv"])
def thermostat_furnace_or_boiler_two_stage_none_single_stage(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/two_stage/metadata_heat_pump_electric_backup_two_stage_heat_pump_two_stage.csv"])
def thermostat_hpeb_2_hp_2(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session", params=["../data/two_stage/metadata_none_two_stage_heat_pump_two_stage.csv"])
def thermostat_na_2_hp_2(request):
    thermostats = from_csv(get_data_path(request.param))
    return next(thermostats)

@pytest.fixture(scope="session")
def core_heating_day_set_hpeb_2_hp_2_entire(thermostat_hpeb_2_hp_2):
    return thermostat_hpeb_2_hp_2.get_core_heating_days(method="entire_dataset")[0]

@pytest.fixture(scope="session")
def core_cooling_day_set_hpeb_2_hp_2_entire(thermostat_hpeb_2_hp_2):
    return thermostat_hpeb_2_hp_2.get_core_cooling_days(method="entire_dataset")[0]


@pytest.fixture(scope="session")
def metrics_hpeb_2_hp_2_data():

    data = [{'sw_version': '2.0.0a1',
	'ct_identifier': 'c61badb0e0c0a7e06932de804af43111',
	'heat_type': 'heat_pump_electric_backup',
	'heat_stage': 'two_stage',
	'cool_type': 'heat_pump',
	'cool_stage': 'two_stage',
	'heating_or_cooling': 'cooling_ALL',
	'zipcode': '27313',
	'station': '723170',
	'climate_zone': 'Mixed-Humid',
	'start_date': '2018-01-01T00:00:00',
	'end_date': '2018-12-31T00:00:00',
	'n_days_in_inputfile_date_range': 364,
	'n_days_both_heating_and_cooling': 17,
	'n_days_insufficient_data': 51,
	'n_core_cooling_days': 160,
	'baseline_percentile_core_cooling_comfort_temperature': 70.8,
	'regional_average_baseline_cooling_comfort_temperature': 73.0,
	'percent_savings_baseline_percentile': 8.463791321979913,
	'avoided_daily_mean_core_day_runtime_baseline_percentile': 30.115655373809567,
	'avoided_total_core_day_runtime_baseline_percentile': 4818.504859809531,
	'baseline_daily_mean_core_day_runtime_baseline_percentile': 355.81755537380957,
	'baseline_total_core_day_runtime_baseline_percentile': 56930.808859809535,
	'_daily_mean_core_day_demand_baseline_baseline_percentile': 12.482315175044594,
	'percent_savings_baseline_regional': -9.403622697474983,
	'avoided_daily_mean_core_day_runtime_baseline_regional': -27.99521353986586,
	'avoided_total_core_day_runtime_baseline_regional': -4479.234166378537,
	'baseline_daily_mean_core_day_runtime_baseline_regional': 297.70668646013416,
	'baseline_total_core_day_runtime_baseline_regional': 47633.06983362146,
	'_daily_mean_core_day_demand_baseline_baseline_regional': 10.443747459873368,
	'mean_demand': 11.425838066476988,
	'tau': 8.651198370580325,
	'alpha': 28.505733943105504,
	'mean_sq_err': 2864.5864690277967,
	'root_mean_sq_err': 53.521831704714636,
	'cv_root_mean_sq_err': 0.16432766190407436,
	'mean_abs_pct_err': 0.12404402189116978,
	'mean_abs_err': 40.4013736135956,
	'total_core_cooling_runtime': 52112.304000000004,
	'daily_mean_core_cooling_runtime': 325.7019,
	'core_cooling_days_mean_indoor_temperature': 71.86122395833334,
	'core_cooling_days_mean_outdoor_temperature': 74.37230553125,
	'core_mean_indoor_temperature': 71.86122395833334,
	'core_mean_outdoor_temperature': 74.37230553125},
	{'sw_version': '2.0.0a1',
	'ct_identifier': 'c61badb0e0c0a7e06932de804af43111',
	'heat_type': 'heat_pump_electric_backup',
	'heat_stage': 'two_stage',
	'cool_type': 'heat_pump',
	'cool_stage': 'two_stage',
	'heating_or_cooling': 'heating_ALL',
	'zipcode': '27313',
	'station': '723170',
	'climate_zone': 'Mixed-Humid',
	'start_date': '2018-01-01T00:00:00',
	'end_date': '2018-12-31T00:00:00',
	'n_days_in_inputfile_date_range': 364,
	'n_days_both_heating_and_cooling': 17,
	'n_days_insufficient_data': 51,
	'n_core_heating_days': 116,
	'baseline_percentile_core_heating_comfort_temperature': 70.0,
	'regional_average_baseline_heating_comfort_temperature': 69,
	'percent_savings_baseline_percentile': 9.514796083459109,
	'avoided_daily_mean_core_day_runtime_baseline_percentile': 33.699673877299716,
	'avoided_total_core_day_runtime_baseline_percentile': 3909.162169766767,
	'baseline_daily_mean_core_day_runtime_baseline_percentile': 354.18177732557547,
	'baseline_total_core_day_runtime_baseline_percentile': 41085.08616976676,
	'_daily_mean_core_day_demand_baseline_baseline_percentile': 13.718239201708988,
	'percent_savings_baseline_regional': 3.147418518670501,
	'avoided_daily_mean_core_day_runtime_baseline_regional': 10.414707505654109,
	'avoided_total_core_day_runtime_baseline_regional': 1208.1060706558767,
	'baseline_daily_mean_core_day_runtime_baseline_regional': 330.89681095392996,
	'baseline_total_core_day_runtime_baseline_regional': 38384.03007065588,
	'_daily_mean_core_day_demand_baseline_baseline_regional': 12.816361242594352,
	'mean_demand': 12.412976715425232,
	'tau': 13.024456363883589,
	'alpha': 25.81831181959944,
	'mean_sq_err': 6287.119332008238,
	'root_mean_sq_err': 79.29135723399013,
	'cv_root_mean_sq_err': 0.2474127459251007,
	'mean_abs_pct_err': 0.18942391878742224,
	'mean_abs_err': 60.70697593640847,
	'total_core_heating_runtime': 37175.924000000006,
	'daily_mean_core_heating_runtime': 320.4821034482759,
	'core_heating_days_mean_indoor_temperature': 68.65607040229885,
	'core_heating_days_mean_outdoor_temperature': 43.64093413793104,
	'core_mean_indoor_temperature': 68.65607040229885,
	'core_mean_outdoor_temperature': 43.64093413793104,
	'total_auxiliary_heating_core_day_runtime': 1512.6000000000001,
	'total_emergency_heating_core_day_runtime': 0.0,
	'rhu1_00F_to_05F': nan,
	'rhu1_05F_to_10F': nan,
	'rhu1_10F_to_15F': nan,
	'rhu1_15F_to_20F': nan,
	'rhu1_20F_to_25F': 0.5963729624072203,
	'rhu1_25F_to_30F': 0.11484128153249978,
	'rhu1_30F_to_35F': 0.07377455861954366,
	'rhu1_35F_to_40F': 0.04290403220944863,
	'rhu1_40F_to_45F': 0.03502547046482063,
	'rhu1_45F_to_50F': 0.013218695583036512,
	'rhu1_50F_to_55F': 0.0,
	'rhu1_55F_to_60F': 0.014154065085746858,
	'rhu1_30F_to_45F': 0.048018860280357595,
	'rhu2_00F_to_05F': nan,
	'rhu2_05F_to_10F': nan,
	'rhu2_10F_to_15F': nan,
	'rhu2_15F_to_20F': nan,
	'rhu2_20F_to_25F': 0.5963729624072203,
	'rhu2_25F_to_30F': nan,
	'rhu2_30F_to_35F': 0.07377455861954366,
	'rhu2_35F_to_40F': 0.04290403220944863,
	'rhu2_40F_to_45F': 0.03502547046482063,
	'rhu2_45F_to_50F': 0.013218695583036512,
	'rhu2_50F_to_55F': 0.0,
	'rhu2_55F_to_60F': nan,
	'rhu2_30F_to_45F': 0.048018860280357595}]

    return data



import pandas as pd
import numpy as np
from numpy.testing import assert_allclose

import tempfile

import pytest

from thermostat.exporters import metrics_to_csv
from thermostat.multiple import multiple_thermostat_calculate_epa_field_savings_metrics

from .fixtures.two_stage_ert import (
        thermostat_ert_hpeb_2_hp_2,
        # thermostat_type_2,
        thermostat_ert_fu_2_ce_2,
        thermostat_ert_fu_2_na_1,
        thermostat_ert_na_2_hp_2,
        metrics_ert_hpeb_2_hp_2_data,
        )
from thermostat.columns import EXPORT_COLUMNS
import six

@pytest.fixture(scope="session")
def metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage(thermostat_ert_hpeb_2_hp_2):
    metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage = thermostat_ert_hpeb_2_hp_2.calculate_epa_field_savings_metrics()
    return metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage

@pytest.fixture(scope="session")
def metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage_multiple(thermostat_ert_hpeb_2_hp_2):
    metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage = multiple_thermostat_calculate_epa_field_savings_metrics([thermostat_ert_hpeb_2_hp_2])
    return metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage

RTOL = 1e-3
ATOL = 1e-3

def test_calculate_epa_field_savings_metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage(metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage, metrics_ert_hpeb_2_hp_2_data):
    assert len(metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage) == len(metrics_ert_hpeb_2_hp_2_data)

    for key in metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage[0].keys():
        test_value = metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage[0][key]
        target_value = metrics_ert_hpeb_2_hp_2_data[0][key]
        if isinstance(test_value, six.string_types):
            assert test_value == target_value
        else:
            assert_allclose(test_value, target_value, rtol=RTOL, atol=ATOL)

    for key in metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage[1].keys():
        test_value = metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage[1][key]
        target_value = metrics_ert_hpeb_2_hp_2_data[1][key]
        if isinstance(test_value, six.string_types):
            assert test_value == target_value
        else:
            assert_allclose(test_value, target_value, rtol=RTOL, atol=ATOL)

def test_multiple_thermostat_calculate_epa_field_savings_metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage(metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage_multiple, metrics_ert_hpeb_2_hp_2_data):
    # Test multiprocessing thermostat code
    assert len(metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage_multiple) == len(metrics_ert_hpeb_2_hp_2_data)

    for key in metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage_multiple[0].keys():
        test_value = metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage_multiple[0][key]
        target_value = metrics_ert_hpeb_2_hp_2_data[0][key]
        if isinstance(test_value, six.string_types):
            assert test_value == target_value
        else:
            assert_allclose(test_value, target_value, rtol=RTOL, atol=ATOL)

    for key in metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage_multiple[1].keys():
        test_value = metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage_multiple[1][key]
        target_value = metrics_ert_hpeb_2_hp_2_data[1][key]
        if isinstance(test_value, six.string_types):
            assert test_value == target_value
        else:
            assert_allclose(test_value, target_value, rtol=RTOL, atol=ATOL)

def test_calculate_epa_field_savings_metrics_type_3(thermostat_ert_fu_2_ce_2):
    metrics_type_3 = thermostat_ert_fu_2_ce_2.calculate_epa_field_savings_metrics()
    assert len(metrics_type_3) == 2

def test_calculate_epa_field_savings_metrics_type_4(thermostat_ert_fu_2_na_1):
    metrics_type_4 = thermostat_ert_fu_2_na_1.calculate_epa_field_savings_metrics()
    assert len(metrics_type_4) == 1

def test_calculate_epa_field_savings_metrics_type_5(thermostat_ert_na_2_hp_2):
    metrics_type_5 = thermostat_ert_na_2_hp_2.calculate_epa_field_savings_metrics()
    assert len(metrics_type_5) == 1

def test_metrics_to_csv(metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage):

    fd, fname = tempfile.mkstemp()
    df = metrics_to_csv(metrics_ert_heat_pump_electric_backup_two_stage_heat_pump_two_stage, fname)

    assert isinstance(df, pd.DataFrame)
    assert df.columns[0] == "sw_version"
    assert df.columns[1] == "ct_identifier"

    with open(fname,'r') as f:
        lines = f.readlines()
        assert len(lines) == 3
        column_heads = lines[0].strip().split(',')
        assert column_heads == EXPORT_COLUMNS



import pandas as pd
import numpy as np
from numpy.testing import assert_allclose

import tempfile

import pytest

from thermostat.exporters import metrics_to_csv
from thermostat.multiple import multiple_thermostat_calculate_epa_field_savings_metrics

from .fixtures.single_stage import (
        thermostat_type_1,
        thermostat_type_2,
        thermostat_type_3,
        thermostat_type_4,
        thermostat_type_5,
        metrics_type_1_data,
        )
from thermostat.columns import EXPORT_COLUMNS
import six

@pytest.fixture(scope="session")
def metrics_type_1(thermostat_type_1):
    metrics_type_1 = thermostat_type_1.calculate_epa_field_savings_metrics()
    return metrics_type_1

@pytest.fixture(scope="session")
def metrics_type_1_multiple(thermostat_type_1):
    metrics_type_1 = multiple_thermostat_calculate_epa_field_savings_metrics([thermostat_type_1])
    return metrics_type_1

RTOL = 1e-3
ATOL = 1e-3

def test_calculate_epa_field_savings_metrics_type_1(metrics_type_1, metrics_type_1_data):
    assert len(metrics_type_1) == len(metrics_type_1_data)

    for key in metrics_type_1[0].keys():
        test_value = metrics_type_1[0][key]
        target_value = metrics_type_1_data[0][key]
        if isinstance(test_value, six.string_types):
            assert test_value == target_value
        else:
            assert_allclose(test_value, target_value, rtol=RTOL, atol=ATOL)

    for key in metrics_type_1[1].keys():
        test_value = metrics_type_1[1][key]
        target_value = metrics_type_1_data[1][key]
        if isinstance(test_value, six.string_types):
            assert test_value == target_value
        else:
            assert_allclose(test_value, target_value, rtol=RTOL, atol=ATOL)

def test_multiple_thermostat_calculate_epa_field_savings_metrics_type_1(metrics_type_1_multiple, metrics_type_1_data):
    # Test multiprocessing thermostat code
    assert len(metrics_type_1_multiple) == len(metrics_type_1_data)

    for key in metrics_type_1_multiple[0].keys():
        test_value = metrics_type_1_multiple[0][key]
        target_value = metrics_type_1_data[0][key]
        if isinstance(test_value, six.string_types):
            assert test_value == target_value
        else:
            assert_allclose(test_value, target_value, rtol=RTOL, atol=ATOL)

    for key in metrics_type_1_multiple[1].keys():
        test_value = metrics_type_1_multiple[1][key]
        target_value = metrics_type_1_data[1][key]
        if isinstance(test_value, six.string_types):
            assert test_value == target_value
        else:
            assert_allclose(test_value, target_value, rtol=RTOL, atol=ATOL)

def test_calculate_epa_field_savings_metrics_type_2(thermostat_type_2):
    metrics_type_2_entire = thermostat_type_2.calculate_epa_field_savings_metrics()
    assert len(metrics_type_2_entire) == 2

    metrics_type_2_yearly = thermostat_type_2.calculate_epa_field_savings_metrics(
            core_cooling_day_set_method="year_end_to_end",
            core_heating_day_set_method="year_mid_to_mid")
    assert len(metrics_type_2_yearly) == 9

def test_calculate_epa_field_savings_metrics_type_3(thermostat_type_3):
    metrics_type_3 = thermostat_type_3.calculate_epa_field_savings_metrics()
    assert len(metrics_type_3) == 2

def test_calculate_epa_field_savings_metrics_type_4(thermostat_type_4):
    metrics_type_4 = thermostat_type_4.calculate_epa_field_savings_metrics()
    assert len(metrics_type_4) == 1

def test_calculate_epa_field_savings_metrics_type_5(thermostat_type_5):
    metrics_type_5 = thermostat_type_5.calculate_epa_field_savings_metrics()
    assert len(metrics_type_5) == 1

def test_metrics_to_csv(metrics_type_1):

    fd, fname = tempfile.mkstemp()
    df = metrics_to_csv(metrics_type_1, fname)

    assert isinstance(df, pd.DataFrame)
    assert df.columns[0] == "sw_version"
    assert df.columns[1] == "ct_identifier"

    with open(fname,'r') as f:
        lines = f.readlines()
        assert len(lines) == 3
        column_heads = lines[0].strip().split(',')
        assert column_heads == EXPORT_COLUMNS
